#!/usr/bin/perl

# Este es un script que visualiza el diagrama de
# Delaunay y de Voronoi. El script genera un 
# archivo Xfig. Para generar una grfica en eps
# se puede hacer:
# fig2dev -L eps s.fig s.eps
#
# Fraga 15/06/2007

die "Args: archivo_puntos  diagrama_delaunay diagrama_voronoi [min max]\n"
	if $#ARGV < 2;

$fpuntos = $ARGV[0];
$fdelaunay = $ARGV[1];
$fvoronoi = $ARGV[2];

# dibujo o no la triangulacin de Delaunay
# si se pasa como segundo argumento un archivo inexistente
# no lo dibuja
$diadelaunay = 0;

# Almacena los mximos y minimos de los puntos de entrada
$xmax1 = $ymax1 = 0;
$xmin1 = $ymin1 = 1e10;

# Almacena los mximos y minimos del diagrama de Delaunay
$xmax2 = $ymax2 = 0;
$xmin2 = $ymin2 = 1e10;

$mymax = $mymin = 0.0;
if ( $#ARGV == 4 ) {
	 $mymin = $ARGV[3];
	 $mymax = $ARGV[4];
}

open( ARCH, "$fpuntos" ) or die "No se puede abrir archivo $fpuntos [MAXY]\n";

$state=0;
$i=0;
while( <ARCH> ) {
	if ( ~/^#/ ) {
		@datos = split( " " );
		if ( $#datos == 0 && $state==0 ) {
			$state = 1;
			$dim = $datos[0],
		}
		elsif ( $#datos == 0 && $state==1 ) {
			$state = 2;
			$n = $datos[0],
		}
		elsif ( $#datos == 1 && $state==2 ) {
			$x[$i] = $datos[0];
			$y[$i] = $datos[1];
			$i++;
		}
		else {
			die "Error 1 en el archivo de puntos!\n";
		}
	}
}
close( ARCH );

# print "i = $i\n";
# print "n = $n\n";

die "Error 2 en el archivo de puntos!\n" if ( $i!=$n );

# Encontramos los minimos y maximos de los puntos de entrada
for ( $i=1; $i<$n; $i++ ){
	if ( $y[$i] > $ymax1 ) {
		$ymax1 = $y[$i];
	} 
	if ( $y[$i] < $ymin1 ) {
		$ymin1 = $y[$i];
	} 
	if ( $x[$i] > $xmax1 ) {
		$xmax1 = $x[$i];
	} 
	if ( $x[$i] < $xmin1 ) {
		$xmin1 = $x[$i];
	} 
}



# Leo el archivo del diagrama de Voronoi
open( ARCH, "$fvoronoi" ) or die "No se puede abrir archivo $fvoronoi\n";

# Necesito leer los valores de los vrtices de Voronoi 
$state=0;
$i=0;
AQUI: while( <ARCH> ) {
	if ( ~/^#/ ) {
		@datos = split( " " );
		if ( $#datos == 0 && $state==0 ) {
			$state = 1;
			$dim = $datos[0],
		}
		elsif ( $#datos == 2 && $state==1 ) {
			$state = 2;
			$np = $datos[0],
			$nt = $datos[1],
		}
		elsif ( $state==2 ) {
			$xvor[$i] = $datos[0];
			$yvor[$i] = $datos[1];
			$i++;
			if ( $i == $np ) {
				$state = 3;
				last AQUI;
			}
		}
	}
}

# Encontramos los minimos y maximos de los vrtices de Voronoi 
for ( $i=1; $i<$np; $i++ ){
	if ( $yvor[$i] > $ymax2 ) {
		$ymax2 = $yvor[$i];
	} 
	if ( $yvor[$i] < $ymin2 ) {
		$ymin2 = $yvor[$i];
	} 
	if ( $xvor[$i] > $xmax2 ) {
		$xmax2 = $xvor[$i];
	} 
	if ( $xvor[$i] < $xmin2 ) {
		$xmin2 = $xvor[$i];
	} 
}

# Cul es el mayor?
if ( $ymax2 > $ymax1 ) {  $ymax1 = $ymax2; }
if ( $xmax2 > $xmax1 ) {  $xmax1 = $xmax2; }
if ( $ymin2 < $ymin1 ) {  $ymin1 = $ymin2; }
if ( $xmin2 < $xmin1 ) {  $xmin1 = $xmin2; }

if( $mymin != 0.0 ) {
	$ymin1 = $mymin; 
	$xmin1 = $mymin;
} 
if( $mymax != 0.0 ) {
	$ymax1 = $mymax; 
	$xmax1 = $mymax;
} 

# Hago el cambio de coordenadas en Y
for ( $i=0; $i<$n; $i++ ){
	$y[$i] = $ymax1 - $y[$i]; 
}
for ( $i=0; $i<$np; $i++ ){
	$yvor[$i] = $ymax1 - $yvor[$i]; 
}

# Hago el cambio de coordenadas en X
for ( $i=0; $i<$n; $i++ ){
	$x[$i] -= $xmin1; 
}
for ( $i=0; $i<$np; $i++ ){
	$xvor[$i] -= $xmin1; 
}

$ymax1 = $ymax1 - $ymin1; 
$xmax1 = $xmax1 - $xmin1; 

if ( $ymax1 < $xmax1 ) {
	$d = $xmax1; 	
}
else {
	$d = $ymax1; 	
}

$scale = 3599/$d;

# Hago el cambio de escala
for ( $i=0; $i<$n; $i++ ){
	$y[$i] = int( $scale * $y[$i] + 0.5); 
	$x[$i] = int( $scale * $x[$i] + 0.5); 
}
for ( $i=0; $i<$np; $i++ ){
	$yvor[$i] = int( $scale * $yvor[$i] + 0.5); 
	$xvor[$i] = int( $scale * $xvor[$i] + 0.5); 
}

# Imprimos el encabezado de un archivo xfig
print  "#FIG 3.2
Landscape
Center
Inches
Letter  
100.00
Single
-2
";

print "# YMAX = $ymax1, YMIN = $ymin1
# XMAX = $xmax1, XMIN =$xmin1
1200 2
";

for ( $i=0; $i<$n; $i++ ){
# 	print "$x[$i] $y[$i]\n";
	$v = $x[$i] + 20;
	print "1 3 0 1 -1 -1 0 0 20 0.000 1 0.0000 $x[$i] $y[$i] 20 20 $x[$i] $y[$i] $v $y[$i]\n";
}

# Abro el archivo de la triangulacin

open( ARCH1, "$fdelaunay" ) or $diadelunay = 1;

if ( $diadelaunay ) {
	$mstate=0;
	$i=0;
	while( <ARCH1> ) {
		if ( ~/^#/ ) {
			@datos = split( " " );
			if ( $#datos == 0 && $mstate==0 ) {
				$mstate = 1;
				$m = $datos[0],
			}
			elsif ( $mstate==1 ) {
				$n = $datos[0],
				print "2 1 0 1 0 7 50 0 -1 0.000 2 0 -1 0 0 4\n\t";
				for ( $i=0; $i<=$#datos; $i++ ){
					$j = $datos[$i];
					print "$x[$j] $y[$j] "
				}
				$j = $datos[0];
				print "$x[$j] $y[$j]\n"
			}
			else {
				die "Error 1 en el archivo de puntos!\n";
			}
		}
	}
	close ( ARCH1 );
}

while( <ARCH> ) {
	if ( ~/^#/ ) {
		@datos = split( " " );
		if ( $#datos == 0 && $state==0 ) {
			$state = 1;
			$dim = $datos[0],
		}
		elsif ( $#datos == 2 && $state==1 ) {
			$state = 2;
			$np = $datos[0],
			$nt = $datos[1],
		}
		elsif ( $state==2 ) {
				$x[$i] = $datos[0];
				$y[$i] = $datos[1];
				$i++;
			if ( $i == $np ) {
				$state = 3;
			}
		}
		elsif ( $state==3 ) {
			# Cuento el nmero de ceros
			$ceros=0;
			for ( $i=1; $i<=$#datos; $i++ ){
				if ( $datos[$i] == 0 ) {
					$ceros++;
				}
			}
			die "ERROR en ceros!\n" if ( $ceros > 1 );
			$n = $datos[0];
			if ( $ceros ) {
				LINE: for ( $i=1; $i<=$n; $i++ ) {
					if ( $datos[$i] == 0 ) {
						$j = $i;
						last LINE;
					}
				}
				$a = $n - 1 - ($j - 1);

				for ( $i=1; $i<=$n; $i++ ) {
					$j = ( $i - 1 + $a ) % $n;
					$ok[$j] = $datos[$i];
				}
				print "2 1 2 1 0 7 50 0 -1 3.000 2 0 -1 0 0";

				$n--;

				print " $n\n\t";

				for ( $i=0; $i<$n; $i++ ) {
					$j = $ok[$i];
					print "$xvor[$j] $yvor[$j] ";
				}
				print "\n";
			}
			elsif ( $n > 2 ) {
				print "2 1 2 1 0 7 50 0 -1 3.000 2 0 -1 0 0";
				$i = $n + 1;
				print " $i\n\t";
				for ( $i=1; $i<=$datos[0]; $i++ ){
					$j = $datos[$i];
					if ( $j != 0 ) {
						print "$xvor[$j] $yvor[$j] ";
					}
				}
				$i = $datos[1];
				print "$xvor[$i] $yvor[$i] ";
			}
			elsif ( $n==2 ) {
				print "2 1 2 1 0 7 50 0 -1 3.000 2 0 -1 0 0 2\n\t";
				for ( $i=1; $i<=2; $i++ ){
					$j = $datos[$i];
					if ( $j != 0 ) {
						print "$xvor[$j] $yvor[$j] ";
					}
				}
			}
			print "\n";
		}
		else {
			die "Error 1 en el archivo de voronoi!\n";
		}
	}
}
close( ARCH );

